@@ -298,6 +298,36 @@ class AccruedInterestListSchema(Schema):
     counterparty = Address()
 
 
+class MediationFeeSchema(Schema):
+
+    value = BigInteger()
+    from_ = Address(data_key="from")
+    to = Address()
+    transactionHash = HexEncodedBytes(attribute="transaction_hash")
+    timestamp = fields.Integer()
+
+
+class MediationFeesListSchema(Schema):
+
+    mediationFees = fields.Nested(MediationFeeSchema, many=True)
+    user = Address()
+    network = Address()
+
+
+class DebtsSchema(Schema):
+
+    debtor = Address(required=True)
+    value = BigInteger(required=True)
+    maximumClaimableValue = BigInteger(required=True, attribute="claimable_value")
+    claimPath = fields.List(Address(), required=False, attribute="claim_path")
+
+
+class DebtsListInCurrencyNetworkSchema(Schema):
+
+    currencyNetwork = Address(required=True, attribute="currency_network")
+    debts = fields.Nested(DebtsSchema, many=True, attribute="debts_list")
+
+
 class TransferInformationSchema(Schema):
 
     currencyNetwork = Address(required=True, attribute="currency_network")
@@ -342,3 +372,12 @@ class TransferIdentifierSchema(Schema):
 class TransactionIdentifierSchema(Schema):
 
     transactionHash = Hash(required=True)
+
+
+class TransferredSumSchema(Schema):
+
+    sender = Address(required=True)
+    receiver = Address(required=True)
+    startTime = fields.Integer(required=True)
+    endTime = fields.Integer(required=True)
+    value = BigInteger(required=True)
