@@ -1,20 +1,17 @@
 import logging
-import socket
 from typing import List, NamedTuple
 
-import gevent
 from gevent import Greenlet
 
 from .currency_network_events import (
     BalanceUpdateEventType,
-    NetworkFreezeEventType,
     TransferEventType,
     TrustlineRequestCancelEventType,
     TrustlineRequestEventType,
     TrustlineUpdateEventType,
     event_builders,
 )
-from .proxy import Proxy, reconnect_interval
+from .proxy import Proxy
 
 
 class Trustline(NamedTuple):
@@ -51,7 +48,6 @@ class CurrencyNetworkProxy(Proxy):
         )
         # Fixed for now, see contracts
         self.interest_rate_decimals = 2
-        self.is_frozen = self._proxy.functions.isNetworkFrozen().call()
 
     def fetch_users(self) -> List[str]:
         return list(self._proxy.functions.getUsers().call())
@@ -95,25 +91,6 @@ class CurrencyNetworkProxy(Proxy):
                     )
         return result
 
-    def start_listen_on_full_sync(self, function, sync_interval: float):
-        def sync():
-            while True:
-                try:
-                    function(self.gen_graph_representation())
-                    gevent.sleep(sync_interval)
-                except socket.timeout as err:
-                    logger.warning(
-                        "Full sync failed because of timeout, try again: " + str(err)
-                    )
-                    gevent.sleep(reconnect_interval)
-                except socket.error as err:
-                    logger.warning(
-                        "Full sync failed because of error, try again: " + str(err)
-                    )
-                    gevent.sleep(reconnect_interval)
-
-        gevent.Greenlet.spawn(sync)
-
     def start_listen_on_balance(self, on_balance, *, start_log_filter=True) -> Greenlet:
         def log(log_entry):
             on_balance(self._build_event(log_entry))
@@ -165,13 +142,3 @@ class CurrencyNetworkProxy(Proxy):
         return self.start_listen_on(
             TransferEventType, log, start_log_filter=start_log_filter
         )
-
-    def start_listen_on_network_freeze(
-        self, on_network_freeze, *, start_log_filter=True
-    ):
-        def log(log_entry):
-            on_network_freeze(self._build_event(log_entry))
-
-        return self.start_listen_on(
-            NetworkFreezeEventType, log, start_log_filter=start_log_filter
-        )
