@@ -6,6 +6,12 @@ from typing import Any, List, NamedTuple
 
 import networkx as nx
 
+from relay.ethindex_db.sync_updates import (
+    BalanceUpdateFeedUpdate,
+    NetworkFreezeFeedUpdate,
+    NetworkUnfreezeFeedUpdate,
+    TrustlineUpdateFeedUpdate,
+)
 from relay.network_graph.graph_constants import balance_ab, creditline_ab, creditline_ba
 from relay.network_graph.trustline_data import (
     get_balance,
@@ -33,6 +39,11 @@ class NetworkGraphConfig(NamedTuple):
     trustlines: List = []
 
 
+class CapacityPath(NamedTuple):
+    capacity: int
+    path: List
+
+
 class Account(object):
     """account from the view of a"""
 
@@ -475,12 +486,13 @@ class CurrencyNetworkGraph(object):
         default_interest_rate=0,
         custom_interests=False,
         prevent_mediator_interests=False,
+        is_frozen=False,
     ):
-
         self.capacity_imbalance_fee_divisor = capacity_imbalance_fee_divisor
         self.default_interest_rate = default_interest_rate
         self.custom_interests = custom_interests
         self.prevent_mediator_interests = prevent_mediator_interests
+        self.is_frozen = is_frozen
         self.graph = nx.Graph()
 
     def gen_network(self, trustlines: List[Any]):
@@ -612,6 +624,31 @@ class CurrencyNetworkGraph(object):
         if account.can_be_closed():
             self.remove_trustline(a, b)
 
+    def update_from_feed(self, feed_update):
+        if type(feed_update) == TrustlineUpdateFeedUpdate:
+            self.update_trustline(
+                feed_update.from_,
+                feed_update.to,
+                feed_update.creditline_given,
+                feed_update.creditline_received,
+                feed_update.interest_rate_given,
+                feed_update.interest_rate_received,
+                feed_update.is_frozen,
+            )
+        elif type(feed_update) == BalanceUpdateFeedUpdate:
+            self.update_balance(
+                feed_update.from_,
+                feed_update.to,
+                feed_update.value,
+                feed_update.timestamp,
+            )
+        elif type(feed_update) == NetworkFreezeFeedUpdate:
+            self.is_frozen = True
+        elif type(feed_update) == NetworkUnfreezeFeedUpdate:
+            self.is_frozen = False
+        else:
+            raise RuntimeError(f"Got feed update of unexpected type {feed_update}")
+
     def create_edge(self, a, b):
         logger.debug("Create new trustline edge: (%s, %s)", a, b)
         self.graph.add_edge(
@@ -855,7 +892,9 @@ class CurrencyNetworkGraph(object):
 
         return PaymentPath(fee=cost[0], path=path, value=value, fee_payer=fee_payer)
 
-    def find_maximum_capacity_path(self, source, target, max_hops=None, timestamp=0):
+    def find_maximum_capacity_path(
+        self, source, target, max_hops=None, timestamp=0
+    ) -> CapacityPath:
         """
         find a path probably with the maximum capacity to transfer from source to target
         The "imbalance_fee" function not being bijective, only an estimate of the fees can be found from "value + fee"
@@ -885,9 +924,9 @@ class CurrencyNetworkGraph(object):
             nx.NetworkXNoPath,
             KeyError,
         ):  # key error for if source or target is not in graph
-            return 0, []
+            return CapacityPath(capacity=0, path=[])
 
-        return -cost[0], list(path)
+        return CapacityPath(capacity=-cost[0], path=list(path))
 
     def get_balances_along_path(self, path):
         balances = []
