@@ -1,13 +1,14 @@
 import json
 import logging
 import os
-import time
 from collections import defaultdict
 from copy import deepcopy
+from enum import Enum
 from typing import Dict, Iterable, List, NamedTuple, Optional
 
 import eth_account
 import eth_keyfile
+import gevent
 import sqlalchemy
 from eth_utils import is_checksum_address, to_checksum_address
 from sqlalchemy.engine.url import URL
@@ -20,6 +21,7 @@ from relay.blockchain.identity_events import FeePaymentEventType
 from relay.blockchain.identity_proxy import IdentityProxy
 from relay.blockchain.proxy import LogFilterListener
 from relay.ethindex_db import ethindex_db
+from relay.ethindex_db.sync_updates import FeedUpdate, graph_update_getter
 from relay.pushservice.client import PushNotificationClient
 from relay.pushservice.client_token_db import (
     ClientTokenAlreadyExistsException,
@@ -72,10 +74,12 @@ class NetworkInfo(NamedTuple):
     is_frozen: bool
 
 
-CURRENCY_CONTRACT_TYPE = "C"
-EXCHANGE_CONTRACT_TYPE = "E"
-TOKEN_CONTRACT_TYPE = "T"
-UNWETH_CONTRACT_TYPE = "U"
+class ContractTypes(Enum):
+    CURRENCY_NETWORK = "CurrencyNetwork"
+    EXCHANGE = "Exchange"
+    UNWETH = "UnwETH"
+    TOKEN = "Token"
+
 
 all_standard_event_types = (
     currency_network_events.standard_event_types
@@ -85,18 +89,19 @@ all_standard_event_types = (
 )
 all_event_builders: Dict[str, BlockchainEvent] = {}
 all_from_to_types: Dict[str, List[str]] = {}
+all_event_contract_types = [contract_type.value for contract_type in ContractTypes]
 for contract_type, contract in [
-    (CURRENCY_CONTRACT_TYPE, currency_network_events),
-    (EXCHANGE_CONTRACT_TYPE, exchange_events),
-    (UNWETH_CONTRACT_TYPE, unw_eth_events),
-    (TOKEN_CONTRACT_TYPE, token_events),
+    (ContractTypes.CURRENCY_NETWORK, currency_network_events),
+    (ContractTypes.EXCHANGE, exchange_events),
+    (ContractTypes.UNWETH, unw_eth_events),
+    (ContractTypes.TOKEN, token_events),
 ]:
     for key, value in contract.event_builders.items():  # type: ignore
-        key = contract_type + key
+        key = contract_type.value + key
         assert key not in all_event_builders
         all_event_builders[key] = value
     for key, value in contract.from_to_types.items():  # type: ignore
-        key = contract_type + key
+        key = contract_type.value + key
         assert key not in all_from_to_types
         all_from_to_types[key] = value
 
@@ -158,12 +163,17 @@ class TrustlinesRelay:
         """return an EthindexDB instance.
         This is being used from relay.api to query for events.
         """
+        address_to_contract_types: Dict[str, str] = {}
+        for address in self.network_addresses:
+            address_to_contract_types[address] = ContractTypes.CURRENCY_NETWORK.value
+
         return ethindex_db.CurrencyNetworkEthindexDB(
             ethindex_db.connect(""),
             address=network_address,
             standard_event_types=currency_network_events.standard_event_types,
-            event_builders=currency_network_events.event_builders,
+            event_builders=all_event_builders,
             from_to_types=currency_network_events.from_to_types,
+            address_to_contract_types=address_to_contract_types,
         )
 
     def get_ethindex_db_for_token(self, address: str):
@@ -206,7 +216,7 @@ class TrustlinesRelay:
         return address in self.network_addresses
 
     def is_currency_network_frozen(self, address: str) -> bool:
-        return self.currency_network_proxies[address].is_frozen
+        return self.currency_network_graphs[address].is_frozen
 
     def is_trusted_token(self, address: str) -> bool:
         return address in self.token_addresses or address in self.unw_eth_addresses
@@ -228,7 +238,7 @@ class TrustlinesRelay:
             custom_interests=proxy.custom_interests,
             prevent_mediator_interests=proxy.prevent_mediator_interests,
             interest_rate_decimals=proxy.interest_rate_decimals,
-            is_frozen=proxy.is_frozen,
+            is_frozen=graph.is_frozen,
         )
 
     def get_network_infos(self) -> List[NetworkInfo]:
@@ -251,8 +261,6 @@ class TrustlinesRelay:
         start_time: int = 0,
         end_time: int = None,
     ):
-        if not end_time:
-            end_time = int(time.time())
         event_selector = self.get_ethindex_db_for_currency_network(network_address)
         return EventsInformationFetcher(
             event_selector
@@ -260,7 +268,20 @@ class TrustlinesRelay:
             network_address, user_address, counterparty_address, start_time, end_time
         )
 
-    def get_transfer_information_for_tx_hash(self, tx_hash):
+    def get_total_sum_transferred(
+        self,
+        network_address,
+        sender_address,
+        receiver_address,
+        start_time=0,
+        end_time=None,
+    ):
+        event_selector = self.get_ethindex_db_for_currency_network(network_address)
+        return EventsInformationFetcher(event_selector).get_total_sum_transferred(
+            sender_address, receiver_address, start_time, end_time
+        )
+
+    def get_transfer_information_for_tx_hash(self, tx_hash: str):
         fetcher = EventsInformationFetcher(self.get_ethindex_db_for_currency_network())
         return fetcher.get_transfer_details_for_tx(tx_hash)
 
@@ -274,6 +295,36 @@ class TrustlinesRelay:
             tx_hash, event_types=FeePaymentEventType
         )
 
+    def get_earned_mediation_fees(
+        self,
+        network_address,
+        user_address,
+        start_time: int = 0,
+        end_time: Optional[int] = None,
+    ):
+        current_network_graph = self.currency_network_graphs[network_address]
+        empty_graph = CurrencyNetworkGraph(
+            capacity_imbalance_fee_divisor=current_network_graph.capacity_imbalance_fee_divisor,
+            default_interest_rate=current_network_graph.default_interest_rate,
+            custom_interests=current_network_graph.custom_interests,
+            prevent_mediator_interests=current_network_graph.prevent_mediator_interests,
+        )
+        event_selector = self.get_ethindex_db_for_currency_network(network_address)
+        return EventsInformationFetcher(
+            event_selector
+        ).get_earned_mediation_fees_in_between_timestamps(
+            user_address, empty_graph, start_time, end_time
+        )
+
+    def get_debt_list_of_user(self, user_address):
+
+        event_selector = self.get_ethindex_db_for_currency_network()
+        return EventsInformationFetcher(
+            event_selector
+        ).get_debt_lists_in_all_networks_with_path(
+            user_address, currency_network_graphs=self.currency_network_graphs
+        )
+
     def deploy_identity(self, factory_address, implementation_address, signature):
         return self.delegate.deploy_identity(
             factory_address, implementation_address, signature
@@ -365,6 +416,19 @@ class TrustlinesRelay:
         if self.config["delegate"]["enable"]:
             self._start_delegate()
         self._load_addresses()
+        self._start_sync_graphs_via_feed()
+
+    def _start_sync_graphs_via_feed(self):
+        conn = ethindex_db.connect("")
+        updates_getter = graph_update_getter()
+
+        def sync():
+            while True:
+                graph_updates = updates_getter(conn)
+                self._apply_feed_update_on_graph(graph_updates)
+                gevent.sleep(self.config["trustline_index"]["sync_interval"])
+
+        gevent.Greenlet.spawn(sync)
 
     def new_network(self, address: str) -> None:
         assert is_checksum_address(address)
@@ -382,8 +446,18 @@ class TrustlinesRelay:
             prevent_mediator_interests=currency_network_proxy.prevent_mediator_interests,
         )
         self._log_listener.add_proxy(currency_network_proxy)
+        self.fully_sync_graph(address)
         self._start_listen_network(address)
 
+    def fully_sync_graph(self, address):
+        logger.info(f"Fully syncing graph from blockchain state for address: {address}")
+
+        self.currency_network_graphs[address].gen_network(
+            self.currency_network_proxies[address].gen_graph_representation()
+        )
+
+        logger.info(f"Graph fully synced for address: {address}")
+
     def new_exchange(self, address: str) -> None:
         assert is_checksum_address(address)
         if address not in self.exchange_addresses:
@@ -540,31 +614,46 @@ class TrustlinesRelay:
         return events
 
     def get_user_events(
-        self, user_address: str, type: str = None, from_block: int = 0,
+        self,
+        user_address: str,
+        event_type: str = None,
+        from_block: int = 0,
+        contract_type: ContractTypes = None,
     ) -> List[BlockchainEvent]:
+        """
+        Get all events of users for user_address.
+        Filter with from_block, event_type and contract_type
+        """
         assert is_checksum_address(user_address)
         event_types: Optional[List[str]]
-        if type:
-            event_types = [type]
+        if event_type:
+            event_types = [event_type]
         else:
             event_types = None
 
-        contract_types: Dict[str, str] = {}
-        for address in self.network_addresses:
-            contract_types[address] = CURRENCY_CONTRACT_TYPE
-        for address in self.exchange_addresses:
-            contract_types[address] = EXCHANGE_CONTRACT_TYPE
-        for address in self.token_addresses:
-            contract_types[address] = TOKEN_CONTRACT_TYPE
-        for address in self.unw_eth_addresses:
-            contract_types[address] = UNWETH_CONTRACT_TYPE
+        address_to_contract_types: Dict[str, str] = {}
+
+        if contract_type == ContractTypes.CURRENCY_NETWORK or contract_type is None:
+            for address in self.network_addresses:
+                address_to_contract_types[
+                    address
+                ] = ContractTypes.CURRENCY_NETWORK.value
+        if contract_type == ContractTypes.EXCHANGE or contract_type is None:
+            for address in self.exchange_addresses:
+                address_to_contract_types[address] = ContractTypes.EXCHANGE.value
+        if contract_type == ContractTypes.TOKEN or contract_type is None:
+            for address in self.token_addresses:
+                address_to_contract_types[address] = ContractTypes.TOKEN.value
+        if contract_type == ContractTypes.UNWETH or contract_type is None:
+            for address in self.unw_eth_addresses:
+                address_to_contract_types[address] = ContractTypes.UNWETH.value
 
         ethindex = ethindex_db.EthindexDB(
             ethindex_db.connect(""),
             standard_event_types=all_standard_event_types,
             event_builders=all_event_builders,
             from_to_types=all_from_to_types,
-            contract_types=contract_types,
+            address_to_contract_types=address_to_contract_types,
         )
         return ethindex.get_all_contract_events(
             event_types, user_address=user_address, from_block=from_block,
@@ -651,6 +740,16 @@ class TrustlinesRelay:
             from_block=from_block,
         )
 
+    def _apply_feed_update_on_graph(
+        self, feed_update: Iterable[FeedUpdate],
+    ):
+        for update in feed_update:
+            if update.address not in self.currency_network_graphs.keys():
+                logger.warning(f"Got event_feed with unknown network address {update}")
+                continue
+            graph = self.currency_network_graphs[update.address]
+            graph.update_from_feed(update)
+
     def _load_gas_price_settings(self, gas_price_settings: Dict):
         method = gas_price_settings["method"]
         methods = ["fixed", "rpc"]
@@ -705,12 +804,7 @@ class TrustlinesRelay:
 
     def _start_listen_network(self, address):
         assert is_checksum_address(address)
-        graph = self.currency_network_graphs[address]
         proxy = self.currency_network_proxies[address]
-        proxy.start_listen_on_full_sync(
-            _create_on_full_sync(graph),
-            self.config["trustline_index"]["full_sync_interval"],
-        )
         proxy.start_listen_on_balance(
             self._process_balance_update, start_log_filter=False
         )
@@ -724,9 +818,6 @@ class TrustlinesRelay:
         proxy.start_listen_on_trustline_request_cancel(
             self._process_trustline_request_cancel, start_log_filter=False
         )
-        proxy.start_listen_on_network_freeze(
-            self._process_network_freeze, start_log_filter=False
-        )
 
     def _start_push_service(self):
         logger.info("Start pushnotification service")
@@ -758,13 +849,6 @@ class TrustlinesRelay:
         )
 
     def _process_balance_update(self, balance_update_event):
-        graph = self.currency_network_graphs[balance_update_event.network_address]
-        graph.update_balance(
-            balance_update_event.from_,
-            balance_update_event.to,
-            balance_update_event.value,
-            balance_update_event.timestamp,
-        )
         self._publish_trustline_events(
             user1=balance_update_event.from_,
             user2=balance_update_event.to,
@@ -785,12 +869,6 @@ class TrustlinesRelay:
         )
         self._publish_blockchain_event(trustline_request_cancel_event)
 
-    def _process_network_freeze(self, network_freeze_event):
-        logger.info(f"Currency network frozen: {network_freeze_event.network_address}")
-        self.currency_network_proxies[
-            network_freeze_event.network_address
-        ].is_frozen = True
-
     def _generate_trustline_events(self, *, user1, user2, network_address, timestamp):
         events = []
         graph = self.currency_network_graphs[network_address]
@@ -827,15 +905,6 @@ class TrustlinesRelay:
 
     def _process_trustline_update(self, trustline_update_event):
         logger.debug("Process trustline update event: %s", trustline_update_event)
-        graph = self.currency_network_graphs[trustline_update_event.network_address]
-        graph.update_trustline(
-            creditor=trustline_update_event.from_,
-            debtor=trustline_update_event.to,
-            creditline_given=trustline_update_event.creditline_given,
-            creditline_received=trustline_update_event.creditline_received,
-            interest_rate_given=trustline_update_event.interest_rate_given,
-            interest_rate_received=trustline_update_event.interest_rate_received,
-        )
         self._publish_blockchain_event(trustline_update_event)
         self._publish_trustline_events(
             user1=trustline_update_event.from_,
@@ -855,13 +924,6 @@ class TrustlinesRelay:
         self.subjects[event.user].publish(event)
 
 
-def _create_on_full_sync(graph):
-    def update_community(graph_rep):
-        graph.gen_network(graph_rep)
-
-    return update_community
-
-
 def create_engine():
     return sqlalchemy.create_engine(
         URL(
