@@ -1,7 +1,7 @@
 import eth_tester
 import pytest
 from tlbin import load_packaged_contracts
-from tldeploy.core import deploy_network, deploy_networks
+from tldeploy.core import NetworkSettings, deploy_network, deploy_networks
 from tldeploy.identity import MetaTransaction
 
 from relay.blockchain import currency_network_proxy
@@ -10,32 +10,23 @@ EXPIRATION_TIME = 4102444800  # 01/01/2100
 
 
 NETWORK_SETTINGS = [
-    {
-        "name": "Cash",
-        "symbol": "CASH",
-        "decimals": 4,
-        "fee_divisor": 1000,
-        "default_interest_rate": 0,
-        "custom_interests": True,
-        "expiration_time": EXPIRATION_TIME,
-    },
-    {
-        "name": "Work Hours",
-        "symbol": "HOU",
-        "decimals": 4,
-        "fee_divisor": 0,
-        "default_interest_rate": 1000,
-        "custom_interests": False,
-        "expiration_time": EXPIRATION_TIME,
-    },
-    {
-        "name": "Beers",
-        "symbol": "BEER",
-        "decimals": 0,
-        "fee_divisor": 0,
-        "custom_interests": False,
-        "expiration_time": EXPIRATION_TIME,
-    },
+    NetworkSettings(
+        name="Cash",
+        symbol="CASH",
+        fee_divisor=1000,
+        custom_interests=True,
+        expiration_time=EXPIRATION_TIME,
+    ),
+    NetworkSettings(
+        name="Work Hours",
+        symbol="HOU",
+        custom_interests=False,
+        default_interest_rate=1000,
+        expiration_time=EXPIRATION_TIME,
+    ),
+    NetworkSettings(
+        name="Beers", symbol="BEER", decimals=0, expiration_time=EXPIRATION_TIME
+    ),
 ]
 
 
@@ -182,10 +173,10 @@ class CurrencyNetworkProxy(currency_network_proxy.CurrencyNetworkProxy):
         self.cancel_trustline_update(to, from_)
 
     def transfer(self, from_, value, max_fee, path, extra_data=b""):
-        txid = self._proxy.functions.transfer(
+        tx_id = self._proxy.functions.transfer(
             value, max_fee, path, extra_data
         ).transact({"from": from_})
-        self._web3.eth.waitForTransactionReceipt(txid)
+        self._web3.eth.waitForTransactionReceipt(tx_id)
 
     def transfer_on_path(self, value, path, max_fee=None, extra_data=b""):
         if max_fee is None:
@@ -218,10 +209,26 @@ class CurrencyNetworkProxy(currency_network_proxy.CurrencyNetworkProxy):
         return self._proxy.functions.balance(from_, to).call()
 
     def freeze_network(self):
-        self._proxy.functions.freezeNetwork().transact()
+        return self._proxy.functions.testFreezeNetwork().transact()
+
+    def unfreeze_network(self):
+        return self._proxy.functions.testUnfreezeNetwork().transact()
+
+    def increase_debt(self, debtor, creditor, value):
+        tx_id = self._proxy.functions.increaseDebt(creditor, value).transact(
+            {"from": debtor}
+        )
+        self._web3.eth.waitForTransactionReceipt(tx_id)
+        return tx_id
+
+    def get_debt(self, debtor, creditor):
+        return self._proxy.functions.getDebt(debtor, creditor).call()
+
+    def assert_debt_value(self, debtor, creditor, value):
+        assert self.get_debt(debtor, creditor) == value
 
 
-@pytest.fixture()
+@pytest.fixture(scope="session")
 def trustlines(accounts):
     return [
         (accounts[0], accounts[1], 100, 150),
@@ -234,25 +241,27 @@ def trustlines(accounts):
     ]  # (A, B, clAB, clBA)
 
 
-@pytest.fixture()
+@pytest.fixture(scope="session")
 def trustlines_with_interests(accounts):
     return [
-        (accounts[0], accounts[1], 1234, 1234, 2000, 1000),
-        (accounts[1], accounts[2], 1234, 1234, 2000, 1000),
-        (accounts[2], accounts[3], 1234, 1234, 2000, 1000),
-        (accounts[3], accounts[4], 1234, 1234, 2000, 1000),
-        (accounts[0], accounts[4], 1234, 1234, 2000, 1000),
+        (accounts[0], accounts[1], 12345, 12345, 2000, 1000),
+        (accounts[1], accounts[2], 12345, 12345, 2000, 1000),
+        (accounts[2], accounts[3], 12345, 12345, 2000, 1000),
+        (accounts[3], accounts[4], 12345, 12345, 2000, 1000),
+        (accounts[0], accounts[4], 12345, 12345, 2000, 1000),
     ]  # (A, B, clAB, clBA)
 
 
 def deploy_test_network(web3):
     return deploy_network(
         web3,
-        "Trustlines",
-        "T",
-        6,
-        fee_divisor=100,
-        expiration_time=EXPIRATION_TIME,
+        network_settings=NetworkSettings(
+            fee_divisor=100,
+            name="Trustlines",
+            symbol="T",
+            custom_interests=True,
+            expiration_time=EXPIRATION_TIME,
+        ),
         currency_network_contract_name="TestCurrencyNetwork",
     )
 
@@ -286,13 +295,18 @@ def testnetwork1_address(web3):
     return deploy_test_network(web3).address
 
 
-@pytest.fixture()
+@pytest.fixture(scope="session")
 def testnetwork2_address(web3):
     return deploy_test_network(web3).address
 
 
-@pytest.fixture()
-def testnetwork3_address(web3):
+@pytest.fixture(scope="session")
+def testnetwork3_address(web3, chain):
+    return deploy_test_network(web3).address
+
+
+@pytest.fixture(scope="session")
+def testnetwork4_address(web3):
     return deploy_test_network(web3).address
 
 
@@ -366,6 +380,32 @@ def currency_network_with_trustlines_and_interests(
     return currency_network
 
 
+@pytest.fixture(scope="session")
+def currency_network_with_trustlines_and_interests_session(
+    web3, currency_network_abi, testnetwork3_address, trustlines_with_interests
+):
+    currency_network = CurrencyNetworkProxy(
+        web3, currency_network_abi, testnetwork3_address
+    )
+    currency_network.setup_trustlines_with_interests_with_updates(
+        trustlines_with_interests
+    )
+
+    return currency_network
+
+
+@pytest.fixture(scope="session")
+def currency_network_with_trustlines_session(
+    web3, currency_network_abi, testnetwork4_address, trustlines
+):
+    currency_network = CurrencyNetworkProxy(
+        web3, currency_network_abi, testnetwork4_address
+    )
+    currency_network.setup_trustlines(trustlines)
+
+    return currency_network
+
+
 @pytest.fixture()
 def address_oracle(testnetworks):
     class AddressOracle:
